/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * Trivial application to use cepstral theta tts
 * 
 * Copyright (C) 2004, Andy Powell
 *
 * Andy Powell <asterisk@automated.it>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 */
 
#include <asterisk/lock.h>
#include <asterisk/file.h>
#include <asterisk/logger.h>
#include <asterisk/options.h>
#include <asterisk/channel.h>
#include <asterisk/pbx.h>
#include <asterisk/module.h>
#include <asterisk/translate.h>
#include <asterisk.h>
#include <string.h>
#include <stdlib.h>
#include <pthread.h>

#include <theta.h>

static char *tdesc = "Trivial Cepstral TTS Application";

static char *app = "Cepstral";

static char *synopsis = "TTS conversion and playback";

static char *descrip = 
"Cepstral(text):  Converts text to speech using Cepstral Theta engine.\n"
"Currently only allows the use of a single voice which is the default\n"
"voice for your install. \n\nUsage example:\n\nexten=> 1,1,Cepstral(\"hello world\")\n\n";

STANDARD_LOCAL_USER;

LOCAL_USER_DECL;

static int mytheta_create_file(char *sayit, char *fname)
{
	theta_voice_desc *vlist, *vdesc;
	cst_voice *vox;

	/* Call this before anything else. */
	theta_init(NULL);
	/* Get all the voices on the system. */
	if(option_verbose > 3)
		ast_verbose(VERBOSE_PREFIX_4 "Checking voices\n");

	if ((vlist = theta_enum_voices(NULL, NULL)) == NULL)
	{
		if(option_verbose > 3)
			ast_verbose(VERBOSE_PREFIX_4 "No voices were found."
				"You may need to set the THETA_VOXPATH environment variable\n");
		return 1;
	}

	vdesc = vlist;
	
	if ((vox = theta_load_voice(vdesc)) == NULL) 
	{
		ast_log(LOG_NOTICE,"Failed to load voice %s\n", vdesc->voxname);
		return 1;
	}

	theta_set_outfile(vox, fname, "riff", 8000, 1);
	theta_tts(sayit, vox);
	theta_unload_voice(vox);
	theta_free_voicelist(vlist);
	return 0;
}

static int cepstral_exec(struct ast_channel *chan, void *data)
{
	int res = 0;
	struct localuser *u;
	char tmp[256];
	char text[256];
	char *c;
	char options[20];
	int noanswer = 0;
	char *stringp=NULL;
	char sfname[256];
	char nsfname[256];

	LOCAL_USER_ADD(u);
	
	if (!data || !strlen((char *)data)) {
		ast_log(LOG_WARNING, "Cepstral requires an argument (text)\n");
		return -1;
	}

	stringp=(char*)data;
	c = strsep(&stringp, "|");
	/* get our text here */
	strncpy(text, c, sizeof(text)-1);
	/* find our options here */
	while(c && strlen(c)) {
		if (!strcasecmp(c, "noanswer"))
			noanswer = 1;
		c = strsep(&stringp, "|");
        }
	/* noanswer = 1 we don't do this */
	if (chan->_state != AST_STATE_UP && noanswer == 0) {
		res = ast_answer(chan);
        
	}

	ast_stopstream(chan);
	pbx_substitute_variables_helper(chan, "${UNIQUEID}", options, sizeof(options) - 1);

	snprintf(sfname, sizeof(sfname)-1, "/tmp/%s.wav", options);
	snprintf(nsfname, sizeof(nsfname)-1, "/tmp/%s", options);

	res = mytheta_create_file(text,sfname);
	if(res == 0)
	{
		res = ast_safe_sleep(chan, 1000);
		if (!res)
		{
			res = ast_streamfile(chan, nsfname, chan->language);
		}
		if (!res)
		{
			res = ast_waitstream(chan, "");
		}
		else
		{
			ast_log(LOG_WARNING, "ast_streamfile failed on %s for %s\n", chan->name, (char *)data);
			res = 0;
		}
		ast_stopstream(chan);
	}
	
	LOCAL_USER_REMOVE(u);
	ast_filedelete(nsfname,"wav");
	return res;
}

int unload_module(void)
{
	STANDARD_HANGUP_LOCALUSERS;
	return ast_unregister_application(app);
}

int load_module(void)
{
	return ast_register_application(app, cepstral_exec, synopsis, descrip);
}

char *description(void)
{
	return tdesc;
}

int usecount(void)
{
	int res;
	STANDARD_USECOUNT(res);
	return res;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}
